@@ -1,21 +1,27 @@
 package Gimp::Data;
 
-use strict;
 use Carp;
-use Tie::Hash;
-use Gimp qw();
-use Exporter ();
-use vars qw(@EXPORT @ISA);
-use base qw(Tie::StdHash Exporter);
+use Gimp ();
 
-@EXPORT = qw();
+sub TIEHASH {
+   my $pkg = shift;
+   my $self;
+
+   bless \$self, $pkg;
+}
 
 sub FETCH {
-    Gimp->gimp_get_data ($_[1]);
+   eval { Gimp->find_parasite ($_[1])->data }
+    || ($@ ? Gimp->get_data ($_[1]) : ());
 }
      
 sub STORE {
-    Gimp->gimp_set_data ($_[1], $_[2]);
+   eval { Gimp->attach_parasite ([$_[1], Gimp::PARASITE_PERSISTENT, $_[2]]) };
+   Gimp->set_data ($_[1], $_[2]) if $@;
+}
+
+sub EXISTS {
+   $_[0]->FETCH ? 1 : ();
 }
      
 tie (%Gimp::Data, 'Gimp::Data');
@@ -47,14 +53,19 @@ values again (L<Gimp::Fu> does this already).
 =head1 %Gimp::Data
 
 You can store and retrieve anything you like in this hash. It's contents
-will automatically be stored in Gimp, and can be accessed in later invocations
-of your plug-in. Be aware that other plug-ins store data in the same "hash", so
-better prefix your key with something unique, like your plug-in's name.
+will automatically be stored in Gimp, and can be accessed in later
+invocations of your plug-in. Be aware that other plug-ins store data
+in the same "hash", so better prefix your key with something unique,
+like your plug-in's name. As an example, the Gimp::Fu module uses
+"function_name/_fu_data" to store its data.
+
+This module might use a persistant implementation, i.e. your data might
+survive a restart of the Gimp application, but you cannot count on this.
 
 =head1 LIMITATIONS
 
-You cannot store references, and you cannot iterate through
-the keys (with C<keys>, C<values> or C<each>).
+You cannot store references, and you cannot (yet) iterate through the keys
+(with C<keys>, C<values> or C<each>).
 
 =head1 AUTHOR
 
